////////////////////////////////////////////////////////////////////
//                                                                //
//              Copyright (c) 1992   Namir Clement Shammas        //
//                                                                //
//  Version:  1.0.0                                  DATE 5/5/92  //
//                                                                //
//  Purpose: implements a class of doubly-linked lists with       //
//           the following operations and features:               //
//                                                                //
//             + insert new node.                                 //
//             + search for a node with a specific occurrence     //
//               of a key.                                        //
//             + delete a node with a specific occurrence of a    //
//               key.                                             //
//             + traverse the nodes of the lists.                 //
//                                                                //
////////////////////////////////////////////////////////////////////


template<class T>
class PDoubleList : DoubleList<void*>
{
  public:
    // ************** state manipulation methods **************
    PDoubleList() : DoubleList<void*>() {} 
    PDoubleList(PDoubleList<T>& dl)
      { DoubleList<void*>::copy(dl); }
    // add virtual ~DoubleList()
    virtual ~PDoubleList()
      { clear(); }

    // ****************** state query methods *****************
    bool isListEmpty() const
      { return DoubleList<void*>::isListEmpty(); }
    unsigned getNodeCount() const
      { return DoubleList<void*>::getNodeCount(); }

    // ************** object manipulation methods *************
    virtual bool insert(T& t1) 
	{ return DoubleList<void*>::insert((void*&)t1); }
    virtual bool insert(T& t1, T& t2) 
	{ return DoubleList<void*>::insert(t1, t2); }
    virtual bool remove(T& x, unsigned occur = 1)
	{ return DoubleList<void*>::remove(x, occur); }
    virtual bool search(T& x, unsigned occur = 1)
	{ return DoubleList<void*>::search(x, occur); }
    virtual bool getFirstNode(T& x)
	{ return DoubleList<void*>::getFirstNode(x); }
    bool getPrevNode(T& x)
	{ return DoubleList<void*>::getPrevNode(x); }
    bool getPrevNode(T x, T& y)
	{ return DoubleList<void*>::getPrevNode(x, y); }
    virtual bool getNextNode(T& x)
	{ return DoubleList<void*>::getNextNode(x); }
    virtual bool getNextNode(T x, T& y)
	{ return DoubleList<void*>::getNextNode(x, y); }
    bool getLastNode(T& x)
	{ return DoubleList<void*>::getLastNode(x); }
    bool getData(T& x, 
                    unsigned index = 0,
                    bool updateNodePtr = false)
	{ return DoubleList<void*>::getData(x, index, updateNodePtr); }
    virtual void clear() 
	{ return DoubleList<void*>::clear(); }
    PDoubleList<T>& operator =(PDoubleList<T>& dl)
      { DoubleList<void*>::copy(dl); return *this; }
};

template<class T>
class PSortedDoubleList  : public PDoubleList<T>
{
  public:
    // ************** state manipulation methods **************
    PSortedDoubleList () : PDoubleList<T>() {}

    PSortedDoubleList (PSortedDoubleList <T>& dl)
      { headPtr = NULL; copy(dl); }

    // ************** object manipulation methods *************
    virtual bool insert(T&);
    virtual bool search(T& x, unsigned occur = 1);

    PSortedDoubleList <T>& operator =(PSortedDoubleList <T>& dl)
      { PDoubleList<T>::copy(dl); return *this; }
};

