#include <string.h>
#include "queue.h"

template<class T>
Queue<T>::Queue()
{
    length = 0;
    allocateError = false;
    head = tail = NULL;
}

template<class T>
Queue<T>& Queue<T>::copy(Queue<T>& g)
{
    QueueNode *p, *q, *r;

    // clear the queue if not already empty
    if (head)
        clear();
    // copy the queue length 
    length = g.length;
    allocateError = false;
    // assign NULL to head and tail of the target queue
    head= tail = NULL;
    // exit if the source queue is empty
    if (!g.head)
        return *this;
    // allocate a new queue element
    head = new QueueNode;
    // exit if error in allocation
    if (!head) {
        allocateError = true;
        return *this;
    }
    // initialize pointers for deep copy
    head->nextPtr = NULL;
    head->nodeData = g.head->nodeData;
    q = g.head->nextPtr;
    p = head;
    // loop to duplicate the source queue elements
    while (q) {
        // create new stack element
        r = new QueueNode;
        if (!r) {
            allocateError = true;
            return *this;
        }
        r->nodeData = q->nodeData; // copy data
        r->nextPtr = NULL;
        p->nextPtr = r;
        p = p->nextPtr;
        q = q->nextPtr;
    }
    tail = p ;
    return *this;
}


template<class T>
void Queue<T>::clear()
{
    T x;

    while (remove(x))
	/* do nothing */;
}

template<class T>
void Queue<T>::add(T x)
{
    QueueNode *p;

    allocateError = false;
    if (tail) {
        p = new QueueNode; // allocate new stack element
        if (!p) {
	    allocateError = true;
	    return;
	}
        p->nodeData = x;
	p->nextPtr = NULL ;
	tail->nextPtr = p ;
	tail = p ;
    }
    else {
        head = new QueueNode;
	if (head == NULL) {
	    allocateError = true;
	    return;
	}
	head->nodeData = x;
	head->nextPtr = NULL;
	tail = head ;
    }
    length++;
}

template<class T>
bool Queue<T>::remove(T& x)
{
    QueueNode *p;

    if (length) {
        x = head->nodeData;
	p = head;
	head = head->nextPtr;
	delete p; // deallocate queue node
	length--;
	if (length == 0)
	    tail = NULL ;
	return true;
    }
    else
	return false;
}


